@@ -27,7 +27,7 @@
  * See the ChangeLog files for a list of changes. 
  */
 
-/* This is a modified version of gtkspell 2.0.2  (gtkspell.sf.net) */
+/* This is a modified version of gtkspell 2.0.5  (gtkspell.sf.net) */
 /* gtkspell - a spell-checking addon for GTK's TextView widget
  * Copyright (c) 2002 Evan Martin.
  */
@@ -48,6 +48,7 @@ struct _GeditAutomaticSpellChecker {
 	
 	GtkTextMark 		*mark_insert;
 	GtkTextTag 		*tag_highlight;
+	GtkTextMark		*mark_click;
 
        	GeditSpellChecker	*spell_checker;
 };
@@ -129,7 +130,17 @@ check_range (GeditAutomaticSpellChecker *spell, GtkTextIter start, GtkTextIter e
 				    spell->tag_highlight, 
 				    &start, 
 				    &end);
-	
+
+	/* Fix a corner case when replacement occurs at beginning of buffer:
+	 * An iter at offset 0 seems to always be inside a word,
+  	 * even if it's not.  Possibly a pango bug.
+	 */
+  	if (gtk_text_iter_get_offset(&start) == 0) 
+	{
+		gtk_text_iter_forward_word_end(&start);
+		gtk_text_iter_backward_word_start(&start);
+	}	
+
 	wstart = start;
 	
 	while (gtk_text_iter_compare (&wstart, &end) < 0) 
@@ -196,9 +207,12 @@ delete_range_after (GtkTextBuffer *buffer, GtkTextIter *start, GtkTextIter *end,
 }
 
 static void
-get_cur_word_extents (GtkTextBuffer *buffer, GtkTextIter *start, GtkTextIter *end) 
+get_word_extents_from_mark (GtkTextBuffer *buffer, 
+			    GtkTextIter   *start, 
+			    GtkTextIter   *end,
+			    GtkTextMark   *mark)
 {
-	gtk_text_buffer_get_iter_at_mark (buffer, start, gtk_text_buffer_get_insert (buffer));
+	gtk_text_buffer_get_iter_at_mark(buffer, start, mark);
 	
 	if (!gtk_text_iter_starts_word (start)) 
 		gtk_text_iter_backward_word_start (start);
@@ -253,8 +267,8 @@ add_to_dictionary (GtkWidget *menuitem, GeditAutomaticSpellChecker *spell)
 	
 	GtkTextIter start, end;
 	
-	get_cur_word_extents (GTK_TEXT_BUFFER (spell->doc), &start, &end);
-	
+	get_word_extents_from_mark (GTK_TEXT_BUFFER (spell->doc), &start, &end, spell->mark_click);	
+
 	word = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (spell->doc), 
 					 &start, 
 					 &end, 
@@ -272,8 +286,8 @@ ignore_all (GtkWidget *menuitem, GeditAutomaticSpellChecker *spell)
 	
 	GtkTextIter start, end;
 	
-	get_cur_word_extents (GTK_TEXT_BUFFER (spell->doc), &start, &end);
-	
+	get_word_extents_from_mark (GTK_TEXT_BUFFER (spell->doc), &start, &end, spell->mark_click);	
+
 	word = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (spell->doc), 
 					 &start, 
 					 &end, 
@@ -293,7 +307,8 @@ replace_word (GtkWidget *menuitem, GeditAutomaticSpellChecker *spell)
 	
 	GtkTextIter start, end;
 
-	get_cur_word_extents (GTK_TEXT_BUFFER (spell->doc), &start, &end);
+	get_word_extents_from_mark (GTK_TEXT_BUFFER (spell->doc), &start, &end, spell->mark_click);	
+
 	oldword = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (spell->doc), &start, &end, FALSE);
 	
 	newword =  g_object_get_qdata (G_OBJECT (menuitem), suggestion_id);
@@ -446,7 +461,7 @@ populate_popup (GtkTextView *textview, GtkMenu *menu, GeditAutomaticSpellChecker
 	char *word;
 
 	/* we need to figure out if they picked a misspelled word. */
-	get_cur_word_extents (GTK_TEXT_BUFFER (spell->doc), &start, &end);
+	get_word_extents_from_mark (GTK_TEXT_BUFFER (spell->doc), &start, &end, spell->mark_click);	
 
 	/* if our highlight algorithm ever messes up, 
 	 * this isn't correct, either. */
@@ -516,6 +531,35 @@ clear_session_cb (GeditSpellChecker          *checker,
 {
 	gedit_automatic_spell_checker_recheck_all (spell);
 }
+
+/* When the user right-clicks on a word, they want to check that word.
+ * Here, we do NOT  move the cursor to the location of the clicked-upon word
+ * since that prevents the use of edit functions on the context menu. 
+ */
+static gboolean
+button_press_event (GtkTextView *view, GdkEventButton *event, gpointer data) 
+{
+	if (event->button == 3) 
+	{
+		gint x, y;
+		GtkTextIter iter;
+		
+		GtkTextBuffer *buffer = gtk_text_view_get_buffer (view);
+		GeditAutomaticSpellChecker *spell = (GeditAutomaticSpellChecker*)data;
+
+		gtk_text_view_window_to_buffer_coords (view, 
+				GTK_TEXT_WINDOW_TEXT, 
+				event->x, event->y,
+				&x, &y);
+		
+		gtk_text_view_get_iter_at_location(view, &iter, x, y);
+
+		gtk_text_buffer_move_mark (buffer, spell->mark_click, &iter);
+	}
+
+	return FALSE; /* false: let gtk process this event, too.
+			 we don't want to eat any events. */
+}
 	
 GeditAutomaticSpellChecker *
 gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checker)
@@ -585,8 +629,7 @@ gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checke
 	spell->tag_highlight = gtk_text_buffer_create_tag (
 				GTK_TEXT_BUFFER (doc),
 				"gtkspell-misspelled",
-				"foreground", "red", 
-				"underline", PANGO_UNDERLINE_SINGLE,
+				"underline", PANGO_UNDERLINE_ERROR,
 				NULL);
 
 	gtk_text_tag_set_priority (spell->tag_highlight, 0);
@@ -609,6 +652,21 @@ gedit_automatic_spell_checker_new (GeditDocument *doc, GeditSpellChecker *checke
 					   spell->mark_insert,
 					   &start);
 
+	spell->mark_click = gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
+					"gedit-automatic-spell-checker-click");
+
+	if (spell->mark_click == NULL)
+		spell->mark_click = 
+			gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (doc),
+						     "gedit-automatic-spell-checker-click",
+						     &start, 
+						     TRUE);
+	else
+		gtk_text_buffer_move_mark (GTK_TEXT_BUFFER (doc),
+					   spell->mark_click,
+					   &start);
+
+
 	return spell;
 }
 
@@ -694,11 +752,16 @@ gedit_automatic_spell_checker_attach_view (
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 	g_return_if_fail (gedit_view_get_document (view) == spell->doc);
 
+	g_signal_connect (G_OBJECT (gedit_view_get_gtk_text_view (view)), 
+			  "button-press-event",
+			  G_CALLBACK (button_press_event), 
+			  spell);
+
 	g_signal_connect (G_OBJECT (view), 
 			  "populate-popup",
 			  G_CALLBACK (populate_popup), 
 			  spell);
-
+	
 	g_signal_connect (G_OBJECT (view), 
 			  "destroy",
 			  G_CALLBACK (view_destroy), 
